package edu.northwestern.at.morphadorner.corpuslinguistics.postagger;

/*  Please see the license information at the end of this file. */

import java.util.*;

import edu.northwestern.at.utils.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.adornedword.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.lexicon.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.postagger.guesser.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.postagger.transitionmatrix.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.postagger.trigram.*;
import edu.northwestern.at.morphadorner.corpuslinguistics.tokenizer.*;

/** Default Part of Speech tagger.
 *
 *  <p>
 *  The Penn Treebank part of speech tagger is a trigram tagger using
 *  the Penn Treebank part of speech tag set.
 *  </p>
 */

public class PennTreebankPartOfSpeechTagger
    extends TrigramTagger
    implements PartOfSpeechTagger
{
    /** Create default part of speech tagger.
     *
     *  @throws Exception   when tagger initialization fails.
     */

    public PennTreebankPartOfSpeechTagger()
        throws Exception
    {
        super();
                                //  Get word and suffix lexicon.

        Lexicon lexicon         = new PennTreebankWordLexicon();
        Lexicon suffixLexicon   = new PennTreebankSuffixLexicon();

                                //  Set word lexicon into tagger.

        setLexicon( lexicon );

                                //  Load transition matrix.

        setTransitionMatrix( new PennTreebankTransitionMatrix() );

                                //  Create part of speech guesser.

        PartOfSpeechGuesser guesser = new DefaultPartOfSpeechGuesser();

                                //  Add lexicons to guesser.

        guesser.setWordLexicon( lexicon );

        guesser.setSuffixLexicon( suffixLexicon );

                                //  Set guesser into tagger.

        setPartOfSpeechGuesser( guesser );
    }
}

/*
Copyright (c) 2008, 2013 by Northwestern University.
All rights reserved.

Developed by:
   Academic and Research Technologies
   Northwestern University
   http://www.it.northwestern.edu/about/departments/at/

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal with the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimers.

    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimers in the documentation and/or other materials provided
      with the distribution.

    * Neither the names of Academic and Research Technologies,
      Northwestern University, nor the names of its contributors may be
      used to endorse or promote products derived from this Software
      without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE CONTRIBUTORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
*/



